using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;

using Syncfusion.Diagnostics;
using Syncfusion.Windows.Forms;
using Syncfusion.Windows.Forms.Grid;

namespace ExcelMarker
{
	/// <summary>
	/// Summary description for ExcelMarkerMouseController.
	/// </summary>
	public class ExcelMarkerMouseController : IMouseController
	{
		GridControlBase owner;
		GridRangeInfo activeRange;
		int lastHitTestCode = GridHitTestContext.None;
		const int HitExcelMarker = 101;
		const int delta = 3;
		private DragWindow dragWindow;
		Cursor cursor1;

		public ExcelMarkerMouseController(GridControlBase owner)
		{
			this.owner = owner;
		}

		public string Name 
		{ 
			get
			{
				return "ExcelMarker"; 
			}
		}

		public Cursor Cursor 
		{ 
			get
			{
				if (cursor1 == null)
				{
					System.IO.Stream stream = GetType().Module.Assembly.GetManifestResourceStream("_41388.cross.CUR");
					if (stream != null)
						cursor1 = new Cursor(stream);
					else
						cursor1 = Cursors.Cross;

				}
				// could check latestHitTestCode here if this controller has
				// different HitTest states. Cursor will only be called if 
				// previous call to HitTest was successfull.
				return cursor1;
			}
		}

		public void MouseHoverEnter()
		{
		}

		/// <summary>
		/// User is moving the mouse over the hot-test area
		/// </summary>
		/// <param name="e"></param>
		public void MouseHover(MouseEventArgs e)
		{
		}

		/// <summary>
		/// Called when the hovering ends, either when user has moved mouse away from hittest area
		/// or when the user has pressed a mouse button.
		/// </summary>
		public void MouseHoverLeave(EventArgs e)
		{
		}

		public void MouseDown(MouseEventArgs e)
		{
			if (e.Button == MouseButtons.Left)
			{
				if (e.Clicks == 1)
				{
					// Start automatic scrolling when user drags mouse out of window
					owner.AutoScrolling = ScrollBars.Both;

					// 
					int rowIndex, colIndex;
					Point point = new Point(e.X, e.Y);
					owner.PointToRowCol(point, out rowIndex, out colIndex);
					activeRange = owner.Selections.Ranges.ActiveRange;

					dragWindow = new DragWindow();
					dragWindow.Opacity = .2f;
					dragWindow.BackColor = SystemColors.Highlight;
					dragWindow.Bounds = GetBounds(activeRange);
					dragWindow.ShowWindowTopMost();
				}
			}
			// else if (e.Clicks == 2)
			// DoubleClick
		}

		Rectangle GetBounds(GridRangeInfo range)
		{
			Rectangle bounds = owner.RangeInfoToRectangle(range, GridRangeOptions.None);
			bounds.Intersect(owner.ClientRectangle);
			bounds = owner.RectangleToScreen(bounds);
			return bounds;
		}

		/// <summary>
		/// User has dragged mouse. If mouse is down, set current position.
		/// </summary>
		/// <param name="e"></param>
		/// 
		int r, c;
		public void MouseMove(MouseEventArgs e)
		{
			// Set current position
			int rowIndex, colIndex;
			Point point = new Point(e.X-delta, e.Y-delta);
			try
			{
				owner.PointToRowCol(point, out rowIndex, out colIndex, 0);
				rowIndex =r= Math.Max(owner.TopRowIndex, rowIndex);
				colIndex =c= Math.Max(owner.LeftColIndex, colIndex);
				GridRangeInfo range = GridRangeInfo.Cells(activeRange.Top, activeRange.Left, rowIndex, colIndex);
				dragWindow.Bounds = GetBounds(range);
				dragWindow.ShowWindowTopMost();
			}
			catch
			{
			}
		}

		/// <summary>
		/// User has release mouse button. Stop automatic scrolling.
		/// </summary>
		/// <param name="e"></param>
		public void MouseUp(MouseEventArgs e)
		{
			// Stop automatic scrolling 
			owner.AutoScrolling = ScrollBars.None;

			int rowIndex, colIndex;
			Point point = new Point(e.X, e.Y);
			owner.PointToRowCol(point, out rowIndex, out colIndex);

			dragWindow.Visible = false;
			dragWindow = null;

			if(rowIndex == -1 )
				rowIndex = r;
			if(colIndex == -1)
				colIndex =c;

			GridRangeInfo range = GridRangeInfo.Cells(activeRange.Top, activeRange.Left, rowIndex, colIndex);
			owner.CurrentCell.SetCurrentCellNoActivate(activeRange.Top, activeRange.Left);
			owner.Model.CutPaste.Copy();

			int row = 0;
			int col = 0;

			if (range.GetFirstCell(out row, out col))
			{
				owner.CurrentCell.SetCurrentCellNoActivate(col, row);
				owner.Model.CutPaste.Paste();
				while(range.GetNextCell(ref row, ref col))
				{
					owner.CurrentCell.MoveTo(row, col);
					owner.Model.CutPaste.Paste();
				}
			}
			//owner.Refresh();
			owner.Selections.Add(range); 
		}

		public void CancelMode()
		{
			owner.AutoScrolling = ScrollBars.None;
			dragWindow.Visible = false;
			dragWindow = null;
		}

		public int HitTest(MouseEventArgs e, IMouseController controller)
		{
			lastHitTestCode = GridHitTestContext.None;

			// Our vote has higher priority than other controllers
			if (e.Button == MouseButtons.Left //&& 
				// (controller == null || controller.Name == "ResizeCells")
				)
			{
				GridRangeInfo range = owner.Selections.Ranges.ActiveRange;
				if (!range.IsEmpty)
				{
					Point point = new Point(e.X, e.Y);
					point.Offset(-delta, -delta);
					int rowIndex, colIndex;
					owner.PointToRowCol(point, out rowIndex, out colIndex);

					if (rowIndex == range.Bottom && colIndex == range.Right)
					{
						point.Offset(delta*2+1, delta*2+1);
						int rowIndex2, colIndex2;
						owner.PointToRowCol(point, out rowIndex2, out colIndex2);
						if (rowIndex2 != rowIndex && colIndex2 != colIndex)
							lastHitTestCode = HitExcelMarker;
					}
				}
			}
			return lastHitTestCode;
		}
	}

}
